@@ -1,8 +1,8 @@
-/*  $Id: AfternoonStalkerEngine.cpp,v 1.15 2002/01/19 03:12:50 sarrazip Exp $
+/*  $Id: AfternoonStalkerEngine.cpp,v 1.20 2003/01/04 11:18:16 sarrazip Exp $
     AfternoonStalkerEngine.cpp - A robot-killing video game engine.
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001 Pierre Sarrazin <sarrazip@iname.com>
+    Copyright (C) 2001, 2002, 2003 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -257,7 +257,7 @@ countDirections(const bool d[4])
 
 AfternoonStalkerEngine::AfternoonStalkerEngine(
 			    const string &windowManagerCaption,
-			    bool useSound, bool fullScreen)
+			    bool useSound, bool pedantic, bool fullScreen)
 						throw(int, string)
 
   : GameEngine(Couple(640, 416), windowManagerCaption, fullScreen),
@@ -344,7 +344,7 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
     letterPPressed(false),
     letterPPressedBefore(false),
 
-    theSoundPlayer(NULL)
+    theSoundMixer(NULL)
 {
     blackColor = SDL_MapRGB(theSDLScreen->format, 0, 0, 0);
 
@@ -360,7 +360,7 @@ AfternoonStalkerEngine::AfternoonStalkerEngine(
 	    throw msg;
 	}
 
-	doOneTimeInitializations(useSound);
+	doOneTimeInitializations(useSound, pedantic);
     }
     catch (string &msg)
     {
@@ -458,7 +458,8 @@ getDir(const char *defaultValue, const char *envVarName)
 
 
 void
-AfternoonStalkerEngine::doOneTimeInitializations(bool useSound) throw(string)
+AfternoonStalkerEngine::doOneTimeInitializations(
+				bool useSound, bool pedantic) throw(string)
 /*  Throws an error message in a 'string' if an error occurs.
 */
 {
@@ -476,30 +477,39 @@ AfternoonStalkerEngine::doOneTimeInitializations(bool useSound) throw(string)
 
 
     /*  Initialize sound system and load sound files, if desired.
-	If useSound is false, then theSoundPlayer remains NULL and
-	the SoundPlayer::WavBuffer objects remain uninitialized.
+	If useSound is false, then theSoundMixer remains NULL and
+	the SoundMixer::Chunk objects remain uninitialized.
     */
     if (useSound)
     {
-	theSoundPlayer = new SoundPlayer();  // may throw string
-	if (gunPickupSound.init(pkgsounddir + "gun-pickup.wav") != 0)
-	    throw string("Could not load sound gun-pickup.wav");
-	if (playerBulletSound.init(pkgsounddir + "player-bullet.wav") != 0)
-	    throw string("Could not load sound player-bullet.wav");
-	if (shootingBlanksSound.init(pkgsounddir + "shooting-blanks.wav") != 0)
-	    throw string("Could not load sound shooting-blanks.wav");
-	if (playerHitSound.init(pkgsounddir + "player-hit.wav") != 0)
-	    throw string("Could not load sound player-hit.wav");
-	if (robotBulletSound.init(pkgsounddir + "robot-bullet.wav") != 0)
-	    throw string("Could not load sound robot-bullet.wav");
-	if (robotHitSound.init(pkgsounddir + "robot-hit.wav") != 0)
-	    throw string("Could not load sound robot-hit.wav");
-	if (batKilledSound.init(pkgsounddir + "bat-killed.wav") != 0)
-	    throw string("Could not load sound bat-killed.wav");
-	if (spiderKilledSound.init(pkgsounddir + "spider-killed.wav") != 0)
-	    throw string("Could not load sound spider-killed.wav");
-	if (newLifeSound.init(pkgsounddir + "new-life.wav") != 0)
-	    throw string("Could not load sound new-life.wav");
+	theSoundMixer = new SoundMixer();  // may throw string
+
+	try
+	{
+	    gunPickupSound.init(pkgsounddir + "gun-pickup.wav");
+	    playerBulletSound.init(pkgsounddir + "player-bullet.wav");
+	    shootingBlanksSound.init(pkgsounddir + "shooting-blanks.wav");
+	    playerHitSound.init(pkgsounddir + "player-hit.wav");
+	    robotBulletSound.init(pkgsounddir + "robot-bullet.wav");
+	    robotHitSound.init(pkgsounddir + "robot-hit.wav");
+	    batKilledSound.init(pkgsounddir + "bat-killed.wav");
+	    spiderKilledSound.init(pkgsounddir + "spider-killed.wav");
+	    newLifeSound.init(pkgsounddir + "new-life.wav");
+	}
+	catch (const SoundMixer::Error &e)
+	{
+	    throw e.what();
+	}
+
+	try
+	{
+	    if (pedantic)
+		missedSound.init(pkgsounddir + "missed.wav");
+	}
+	catch (const SoundMixer::Error &e)
+	{
+	    // Don't use this (lame) sound if it is mysteriously absent...
+	}
     }
 }
 
@@ -1425,7 +1435,7 @@ AfternoonStalkerEngine::animateAutomaticCharacters()
     moveEnemyList(spiderSprites, SPIDER_SPEED_FACTOR + incr);
     moveEnemyList(batSprites, BAT_SPEED_FACTOR + incr);
 
-    moveBullets(playerBulletSprites);
+    moveBullets(playerBulletSprites, true);
     moveBullets(robotBulletSprites);
 
 
@@ -1619,7 +1629,13 @@ AfternoonStalkerEngine::moveEnemyList(SpriteList &slist, int speedFactor)
 
 
 void
-AfternoonStalkerEngine::moveBullets(SpriteList &slist)
+AfternoonStalkerEngine::moveBullets(SpriteList &slist, bool playMissedSound)
+/*
+    If 'playMissedSound' is true, then a "missed" sound will be played
+    if the bullet dies without having hit anything.
+    A bullet is considered to have hit something if its "time to live"
+    field is not zero.
+*/
 {
     for (SpriteList::iterator it = slist.begin(); it != slist.end(); it++)
     {
@@ -1671,6 +1687,8 @@ AfternoonStalkerEngine::moveBullets(SpriteList &slist)
 
 	if (!allowed)
 	{
+	    if (playMissedSound && bullet->getTimeToLive() == 0)
+		playSoundEffect(missedSound);
 	    *it = NULL;
 	    delete bullet;
 	}
@@ -1857,6 +1875,9 @@ AfternoonStalkerEngine::detectCollisions()
 		{
 		    playSoundEffect(robotHitSound);
 
+		    bullet->setTimeToLive(1);
+			// Patch: mark this bullet as having hit something.
+
 		    if (--robot->numShotsBeforeDeath == 0)
 		    {
 			createRobotExplosion(*robot);
@@ -1909,6 +1930,9 @@ AfternoonStalkerEngine::detectCollisions()
 	    {
 		playSoundEffect(spiderKilledSound);
 
+		bullet->setTimeToLive(1);
+		    // Patch: mark this bullet as having hit something.
+
 		createAnimalExplosion(spider->getCenterPos(),
 				spiderExplosionPA, spiderExplosionSprites);
 		createScoreSprites(100, spider->getCenterPos());
@@ -1937,6 +1961,9 @@ AfternoonStalkerEngine::detectCollisions()
 	    {
 		playSoundEffect(batKilledSound);
 
+		bullet->setTimeToLive(1);
+		    // Patch: mark this bullet as having hit something.
+
 		createAnimalExplosion(bat->getCenterPos(),
 				    batExplosionPA, batExplosionSprites);
 		createScoreSprites(300, bat->getCenterPos());
@@ -1966,6 +1993,8 @@ AfternoonStalkerEngine::detectCollisions()
 	    if (robotBullet->getPixmapArray() == &bigRobotBulletPA
 				&& bullet->collidesWithSprite(*robotBullet))
 	    {
+		playSoundEffect(missedSound);
+
 		delete bullet;
 		bullet = NULL;
 		*itb = NULL;
@@ -1980,6 +2009,8 @@ AfternoonStalkerEngine::detectCollisions()
 
 	if (bullet != NULL && absorbedByBlinking)
 	{
+	    playSoundEffect(missedSound);
+
 	    delete bullet;
 	    bullet = NULL;
 	    *itb = NULL;
@@ -2469,7 +2500,7 @@ AfternoonStalkerEngine::displayStartMessage(bool display)
 	switch ((tickCount - firstTick) / 32 % 3)
 	{
 	    case 0:
-		msg = "Afternoon Stalker - by Pierre Sarrazin";
+		msg = "Afternoon Stalker " VERSION " - by Pierre Sarrazin";
 		break;
 	    case 1:
 		msg = "Move with arrow keys - shoot with Ctrl key";
@@ -2511,10 +2542,19 @@ AfternoonStalkerEngine::addToNumPlayerLives(int n)
 
 
 void
-AfternoonStalkerEngine::playSoundEffect(SoundPlayer::WavBuffer &wb)
-/*  theSoundPlayer is still NULL if the sound effects were not wanted.
-*/
+AfternoonStalkerEngine::playSoundEffect(SoundMixer::Chunk &wb)
 {
-    if (theSoundPlayer != NULL)
-	theSoundPlayer->playWavBuffer(wb);
+    /*  'theSoundMixer' is still NULL if the sound effects were not wanted.
+    */
+    if (theSoundMixer != NULL)
+    {
+	try
+	{
+	    theSoundMixer->playChunk(wb);
+	}
+	catch (const SoundMixer::Error &e)
+	{
+	    fprintf(stderr, "playSoundEffect: %s\n", e.what().c_str());
+	}
+    }
 }
