@@ -1,8 +1,8 @@
-/*  $Id: AfternoonStalkerEngine.h,v 1.9 2002/01/19 03:12:50 sarrazip Exp $
-AfternoonStalkerEngine.h - A robot-killing video game engine.
+/*  $Id: AfternoonStalkerEngine.h,v 1.15 2003/01/04 11:18:16 sarrazip Exp $
+    AfternoonStalkerEngine.h - A robot-killing video game engine.
 
     afternoonstalker - A robot-killing video game.
-    Copyright (C) 2001 Pierre Sarrazin <sarrazip@iname.com>
+    Copyright (C) 2001, 2002, 2003 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -25,17 +25,13 @@ AfternoonStalkerEngine.h - A robot-killing video game engine.
 
 #define PAUL 12
 
-#define __STDC_LIMIT_MACROS  /* to get SIZE_MAX from <stdint.h> */
-
 #include <gengameng/GameEngine.h>
 #include <gengameng/Sprite.h>
 
-#include "SoundPlayer.h"
+#include "SoundMixer.h"
 
 class RobotSprite;
 
-#include <stdint.h>
-
 #include <string>
 
 using namespace std;
@@ -57,7 +53,7 @@ public:
     */
 
     AfternoonStalkerEngine(const string &windowManagerCaption,
-			    bool useSound, bool fullScreen)
+			    bool useSound, bool pedantic, bool fullScreen)
 						throw(int, string);
     /*  See base class.
 
@@ -244,16 +240,17 @@ private:
 
     /*  SOUND EFFECTS:
     */
-    SoundPlayer *theSoundPlayer;  // see method playSoundEffect()
-    SoundPlayer::WavBuffer gunPickupSound;
-    SoundPlayer::WavBuffer playerBulletSound;
-    SoundPlayer::WavBuffer shootingBlanksSound;
-    SoundPlayer::WavBuffer playerHitSound;
-    SoundPlayer::WavBuffer robotBulletSound;
-    SoundPlayer::WavBuffer robotHitSound;
-    SoundPlayer::WavBuffer batKilledSound;
-    SoundPlayer::WavBuffer spiderKilledSound;
-    SoundPlayer::WavBuffer newLifeSound;
+    SoundMixer *theSoundMixer;  // see method playSoundEffect()
+    SoundMixer::Chunk gunPickupSound;
+    SoundMixer::Chunk playerBulletSound;
+    SoundMixer::Chunk shootingBlanksSound;
+    SoundMixer::Chunk playerHitSound;
+    SoundMixer::Chunk robotBulletSound;
+    SoundMixer::Chunk robotHitSound;
+    SoundMixer::Chunk batKilledSound;
+    SoundMixer::Chunk spiderKilledSound;
+    SoundMixer::Chunk newLifeSound;
+    SoundMixer::Chunk missedSound;
 
 
     ///////////////////////////////////////////////////////////////////////////
@@ -289,7 +286,7 @@ private:
     bool isOutOfSetting(const Sprite &s) const;
     size_t getNumToughRobots() const;
     void moveEnemyList(SpriteList &slist, int speedFactor);
-    void moveBullets(SpriteList &slist);
+    void moveBullets(SpriteList &slist, bool playMissedSound = false);
     int chooseDirectionTowardTarget(Couple startPos,
                                     Couple targetPos,
                                     int speedFactor,
@@ -323,10 +320,10 @@ private:
     void initializePlayerLife();
     void setTimeUntilNextRobot();
     void setTimeUntilNextAnimal();
-    void doOneTimeInitializations(bool useSound) throw(string);
+    void doOneTimeInitializations(bool useSound, bool pedantic) throw(string);
     void loadLevel(int levelNo, const string &setFilename) throw(string);
 
-    void playSoundEffect(SoundPlayer::WavBuffer &wb);
+    void playSoundEffect(SoundMixer::Chunk &wb);
 
 
     /*	Forbidden operations:
